package com.googlecode.mso.gmso2;
import java.io.FileNotFoundException;
import java.util.List;

import org.freedesktop.bindings.Internationalization;
import org.gnome.gdk.Event;
import org.gnome.gdk.Pixbuf;
import org.gnome.glib.Glib;
import org.gnome.gtk.AboutDialog;
import org.gnome.gtk.AttachOptions;
import org.gnome.gtk.Button;
import org.gnome.gtk.Dialog;
import org.gnome.gtk.ErrorMessageDialog;
import org.gnome.gtk.Expander;
import org.gnome.gtk.Fixed;
import org.gnome.gtk.Gtk;
import org.gnome.gtk.IconSize;
import org.gnome.gtk.Image;
import org.gnome.gtk.ImageMenuItem;
import org.gnome.gtk.Label;
import org.gnome.gtk.Menu;
import org.gnome.gtk.MenuBar;
import org.gnome.gtk.MenuItem;
import org.gnome.gtk.ProgressBar;
import org.gnome.gtk.Settings;
import org.gnome.gtk.Stock;
import org.gnome.gtk.Table;
import org.gnome.gtk.TextComboBox;
import org.gnome.gtk.VBox;
import org.gnome.gtk.Widget;
import org.gnome.gtk.Window;
import org.gnome.gtk.WindowPosition;

import com.googlecode.mso.jinklevel.Cartridge;
import com.googlecode.mso.jinklevel.InkLevelException;
import com.googlecode.mso.libmso.LibMso;
import com.googlecode.mso.libmso.Printer;
import static org.freedesktop.bindings.Internationalization._;
import static org.freedesktop.bindings.Internationalization.N_;

public class Gmso2 {

	private List<Printer> printers;

	private static final String[] PRINTER_INFO_LABELS = { N_("Model:"), N_("Serial no:"), N_("Driver:"), N_("Device file:")};
	
	private static final String VERSION = "0.1";

	public Gmso2() {
		final Window window = new Window();
		final VBox box;
		final Fixed fixed;
		final Button scan;
		final Button  query = new Button(_("Query"));
		final TextComboBox printerCombo;
		final Label printer;
		final MenuBar menuBar;
		final MenuItem fileMenuItem, helpMenuItem;
		final Menu fileMenu, helpMenu;
		final ImageMenuItem quit, about;

		menuBar = new MenuBar();

		quit = new ImageMenuItem(new Image(Stock.QUIT,IconSize.MENU),_("Quit"));
		quit.connect
		(
				new MenuItem.Activate() {

					public void onActivate(MenuItem source) {
						Gtk.mainQuit();
					}
				}
		);

		fileMenu = new Menu();
		fileMenu.append(quit);
		fileMenuItem = new MenuItem(_("File"));
		fileMenuItem.setSubmenu(fileMenu);
		menuBar.append(fileMenuItem);

		about = new ImageMenuItem(new Image(Stock.ABOUT,IconSize.MENU),_("About"));
		about.connect
		(
				new MenuItem.Activate() {

					public void onActivate(MenuItem source) {
						AboutDialog about = new AboutDialog();
						about.setComments(_("Queries ink levels of libinklevel supported printers."));
						about.setCopyright("© 2010 Serkan Kaba");
						about.setProgramName("GMSO 2");
						about.setAuthors(new String[] {"Serkan Kaba <serkan_kaba@yahoo.com>"});
						about.setTranslatorCredits(_("translator-credits"));
						about.setArtists(new String[] {"Adem Alp Yıldız <ademalp@gmail.com>"});
						about.setWebsite("http://mso.googlecode.com/");
						about.setWebsiteLabel(_("GMSO 2 Homepage"));
						try {
							about.setLogo(new Pixbuf("/usr/share/pixmaps/gmso2_logo.png"));
						} catch (FileNotFoundException e) {
						}
						about.setVersion(VERSION);
						about.setSkipPagerHint(true);
						about.setSkipTaskbarHint(true);
						about.setModal(true);
						about.showAll();
						about.run();
					}
				}
		);

		helpMenu = new Menu();
		helpMenu.append(about);
		helpMenuItem = new MenuItem(_("Help"));
		helpMenuItem.setSubmenu(helpMenu);
		menuBar.append(helpMenuItem);

		fixed = new Fixed();
		fixed.setSizeRequest(320, 100);

		printer=new Label(_("Printer:"));
		fixed.put(printer, 16, 12);

		printerCombo=new TextComboBox();
		printerCombo.setSizeRequest(200, 28);
		printerCombo.setSensitive(true);
		fixed.put(printerCombo, 96, 6);

		scan = new Button(_("Scan Printers"));
		scan.setImage(new Image(Stock.PRINT_PREVIEW,IconSize.BUTTON));

		fixed.put(scan, 16, 50);
		scan.connect
		(
				new Button.Clicked() {

					public void onClicked(Button source) {
						if(printers != null && printers.size() != 0) {
							for(int i=0;i<printers.size();i++) {
								printerCombo.removeText(i);
							}
						}
						printers=LibMso.listPrinters();
						if(printers==null || printers.size()==0) {
							ErrorMessageDialog dialog= new ErrorMessageDialog(window,_("Error"),_("No printers found"));
							dialog.run();
							query.setSensitive(false);
						}
						else {

							for(Printer printer: printers) {
								printerCombo.appendText(printer.getModel());
							}
							printerCombo.setActive(0);
							query.setSensitive(true);

						}
					}

				}
		);


		query.setSensitive(false);
		query.setImage(new Image(Stock.EXECUTE,IconSize.BUTTON));
		fixed.put(query, 225, 50);
		query.connect
		(
				new Button.Clicked() {

					public void onClicked(Button source) {

						final Dialog dialog=new Dialog();
						final VBox box;
						final Table printerInfoTable,inkLevelInfoTable;
						final Label[] printerInfoLabels;
						final Expander expander;
						final Printer printer;
						
						printer = printers.get(printerCombo.getActive());

						final List<Cartridge> cartridges;

						try {
							cartridges = printer.getCartridges();
						} catch (InkLevelException e) {
							ErrorMessageDialog errorDialog= new ErrorMessageDialog(window,_("Error"),e.getMessage());
							errorDialog.run();
							return;
						}

						box = new VBox(false, 0);

						printerInfoTable = new Table(4,2,false);

						printerInfoLabels = new Label[4];

						for(int i=0;i<PRINTER_INFO_LABELS.length;i++) {
							final Label label = new Label(_(PRINTER_INFO_LABELS[i]));
							printerInfoTable.attach(label, 0, 1, i, i+1,AttachOptions.SHRINK,AttachOptions.FILL,0,0);

							printerInfoLabels[i] = new Label();
							printerInfoTable.attach(printerInfoLabels[i], 1, 2, i, i+1,AttachOptions.SHRINK,AttachOptions.FILL,0,0);
						}

						printerInfoLabels[0].setLabel(printer.getModel());
						printerInfoLabels[1].setLabel(printer.getSerial());
						printerInfoLabels[2].setLabel(printer.getDriver());
						printerInfoLabels[3].setLabel(printer.getDeviceFile());

						box.add(printerInfoTable);

						inkLevelInfoTable = new Table(cartridges.size(),2,false);

						for(int i=0;i<cartridges.size();i++) {
							final Label color = new Label(cartridges.get(i).getColor().toString());
							inkLevelInfoTable.attach(color, 0, 1, i, i+1);

							final ProgressBar percentage = new ProgressBar();
							percentage.setText(Integer.toString(cartridges.get(i).getInkLevel())+"%");
							percentage.setFraction((double)cartridges.get(i).getInkLevel()/100);
							inkLevelInfoTable.attach(percentage, 1, 2, i, i+1);
						}

						expander = new Expander(_("Ink Levels"));
						expander.add(inkLevelInfoTable);

						box.add(expander);

						dialog.setSkipPagerHint(true);
						dialog.setSkipTaskbarHint(true);
						dialog.setResizable(false);

						dialog.add(box);

						dialog.showAll();
						dialog.run();
					}

				}
		);

		box = new VBox(false, 0);
		box.add(menuBar);
		box.add(fixed);
		window.add(box);

		window.setPosition(WindowPosition.CENTER);
		window.setResizable(false);

		window.setTitle("GMSO 2 " + VERSION);
		window.connect(new Window.DeleteEvent() {
			public boolean onDeleteEvent(Widget source, Event event) {
				Gtk.mainQuit();
				return false;
			}
		});
		window.showAll();
	}

	public static void main(String[] args) {
		Glib.setProgramName("gmso2");
		Gtk.init(args);
		Internationalization.init("gmso2", "/usr/share/locale/");
		if(!"root".equals(Glib.getUserName())) {
			ErrorMessageDialog dialog= new ErrorMessageDialog(null,_("Error"),_("GMSO 2 must be run as root"));
			dialog.run();
			System.exit(-1);
		}
		try {
			Gtk.setDefaultIcon(new Pixbuf("/usr/share/pixmaps/gmso2_icon.png"));
		} catch (FileNotFoundException e) {
		}
		new Gmso2();
		Settings settings = Gtk.getSettings();
		settings.setButtonImages(true);
		settings.setMenuImages(true);
		Gtk.main();
	}

}
